#include <thread>
#include <vector>
#include <TFile.h>
#include <TH1D.h>
#include <TH2F.h>
#include <TRandom3.h>

using namespace std;

struct myStruct{
  
  const unsigned nEvents;
  const vector<TH2F> map;
  
  myStruct(const unsigned nEvents, const vector<TH2F>& map):nEvents(nEvents),map(map) {}
  void operator()(){
        
    for(unsigned i = 0; i<nEvents; ++i){

      TH1D proj = *(map.front().ProjectionX("proj",1, 1));
      proj.GetRandom();

    }

  };
  
};

template <class T> void launchThreads(vector<T>& workers){

  unsigned nThreads = thread::hardware_concurrency();//get the number of working cores
//   unsigned nThreads = 1;
  if(nThreads == 0) nThreads += 1;
  vector<thread> threads(nThreads);
  
  auto itWk = workers.begin();
  while(itWk != workers.end()){
    
    auto itTh = threads.begin(); 
    while(itTh != threads.end() && itWk != workers.end()){

      *itTh = thread(*itWk);//start each threads
      ++itTh;
      ++itWk;
      
    }
    
    for(thread& th : threads) th.join();//join them all to the current thread
    
  }
  
}

void generateLiBranches(const unsigned nEvents = 1e4){
  
  vector<myStruct> workers;//speed up the branch generations with a thread per branch
    
  vector<TH2F> betaMapLi (5);
  TFile LiFile("spectres_beta/cartes_li9.root");
  betaMapLi[0] = *static_cast<TH2F*>(LiFile.Get("carte_0"));
  betaMapLi[1] = *static_cast<TH2F*>(LiFile.Get("carte_1"));
  betaMapLi[2] = *static_cast<TH2F*>(LiFile.Get("carte_2"));
  betaMapLi[3] = *static_cast<TH2F*>(LiFile.Get("carte_3"));
  betaMapLi[4] = *static_cast<TH2F*>(LiFile.Get("carte_4"));
  

  workers.push_back(myStruct(nEvents, betaMapLi)); 
  workers.push_back(myStruct(nEvents, betaMapLi)); 
  workers.push_back(myStruct(nEvents, betaMapLi));
  workers.push_back(myStruct(nEvents, betaMapLi)); 
  workers.push_back(myStruct(nEvents, betaMapLi)); 
  workers.push_back(myStruct(nEvents, betaMapLi)); 
  workers.push_back(myStruct(nEvents, betaMapLi)); 

  launchThreads(workers);
  
}

int main(int argc, char* argv[]){
  
  if(argc == 2) generateLiBranches(stod(argv[1]));
  else generateLiBranches();
  return 0;

}
